###########################################################
## R kode til "MATIA sier...", publisert 10. oktober 2023,
## av Genaro Sucarrat (BI) og Erica Dalst (SEB)
##
## Lisens: GNU GPL 2 eller hyere
##
## Sitering av kode:
## Genaro Sucarrat (2023): "R kode til 'MATIA sier...'",
## 10. oktober 2023. https://www.sucarrat.net/matia/
##
##
## R koden i denne filen gjennomfrer en ex-post evaluering
## av om prognosene i Nasjonalbudsjettet er kjennetegnet
## ved systematisk anslagsfeil (dvs. om prognosene er
## systematisk for lave eller for hye)
##
##
## Innhold:
##
## 1 INITIR
## 2 RLIGE DATA: LAST OG TRANSFORMR
## 3 FIN PROGNOSER
## 4 SSB PROGNOSER
## 5 NORGES BANK PROGNOSER
## 6 SAMMENLIGN PROGNOSER
## 7 LAG TABELLER (LaTeX kode)
##
###########################################################

## Background:
##============
##
## EU directive no. 2011/85: 
## "On requirements for budgetary frameworks of the Member States"
## http://data.europa.eu/eli/dir/2011/85/oj
##
## CHAPTER III: FORECASTS, Article 4:
##
## "(1) Member States shall ensure that fiscal planning is based on realistic
## macroeconomic and budgetary forecasts... The macroeconomic and budgetary
## forecasts shall be compared with the most updated forecasts of the
## Commission and, if appropriate, those of other independent bodies"
##
## "(6) The macroeconomic and budgetary forecasts for fiscal planning shall be
## subject to regular, unbiased and comprehensive evaluation based on objective
## criteria, including ex post evaluation".
##
## While the directive does not apply to Norway since she is not member of
## the European Monetary Union, it is of interest to implement the directive.


#####################################################################
## 1 INITIR
#####################################################################

##definer arbeidskatalog p lokal maskin:
setwd("C:/Users/sucarrat/Documents/projects/matia/2023/")
#setwd(choose.dir())

##slett alle objekter i gjeldende arbeidsrom:
rm(list = ls())

##last flgende CRAN pakker:
library(gets)   #trenger zoo()
library(garchx) #trenger gdiff og glag
library(xtable) #for generering av LaTeX kode

###sett noen parametre:
#iaar <- 2023 #i r
#iaarTxt <- as.character(iaar)
#ifjorTxt <- as.character(iaar-1)


#####################################################################
## 2 RLIGE DATA: LAST OG TRANSFORMR
#####################################################################

##last faktiske verdier:
##======================

##last datasett m/faktiske verdier:
actualDataY <-
  read.csv("./analysis/data/actual-data-yearly-2023m9vintage.csv")
rownames(actualDataY) <- as.character(actualDataY$aar)

##hvilke serier?:
seriesNames <- c("bnpfast", "privatkons", "offkons", "investbrutto",
  "sysselsetting", "ledighetaku", "lonnsvekst", "kpi", 
  "nibor3m", "i44", "brentblend")


##lag liste med faktiske serier (transformerte/vekst data):
##=========================================================

seriesListY <- list()
for(i in 1:length(seriesNames)){
  x <- actualDataY[ ,seriesNames[i] ]  
  x <- zoo(x, order.by=actualDataY[,"aar"])
  ##hvis i44:
  if(seriesNames[i] == "i44"){
    x <- 100*gdiff(x)/glag(x) #gdiff og glag er fra lgarch pakka
  }
  ##hvis olje:
  if(seriesNames[i] == "brentblend"){
    nokusd <- actualDataY[ ,"nokusd" ]
    nokusd <- zoo(nokusd, order.by=actualDataY[,"aar"])
    x <- x*nokusd #oljepris i norske kroner
    x <- 100*gdiff(x)/glag(x) #gdiff og glag er fra lgarch pakka
  }
  seriesListY[[i]] <- x
}
names(seriesListY) <- seriesNames


###########################################################
## 3 FIN PROGNOSER
###########################################################
                     
##evalueringsperiode:
years <- 1998:2022

##filnavn:
fileNames <- paste0("forecasts-nasjonalbudsjettet", years+1, ".csv")
fileNames <- ##legg til katalog:
  paste0("./analysis/data/forecasts-historical/nasjonalbudsjettet/",
  fileNames)

##tom matrise m/prognoser:
forecastData <- matrix(NA, length(years), length(seriesNames))                     
colnames(forecastData) <- seriesNames
rownames(forecastData) <- years

##liste m/FIN prognoser (t, t+1 og t+2):
FINforecasts <- list()
FINforecasts$t <- forecastData
FINforecasts$tplus1 <- forecastData
FINforecasts$tplus2 <- forecastData

##lag brentblendnok:
brentblendusd <-
  zoo(actualDataY[,"brentblend"], order.by=actualDataY[,"aar"])
nokusd <- 
  zoo(actualDataY[ ,"nokusd" ], order.by=actualDataY[,"aar"])
brentblendnok <- nokusd*brentblendusd

##lkke:
for(i in 1:length(fileNames)){

  ##last data:
  tmpdata <- read.csv(fileNames[i])

  ##fyll prognoser:
  for(j in 1:length(seriesNames) ){

    ##navn:
    tmpname <- paste0("FIN", seriesNames[j])
    if( seriesNames[j] =="i44" ){ tmpname <- "FINi44vekst" }
    if( seriesNames[j] =="brentblend" ){ tmpname <- "FINbrentblendnok" }

    ##t:
    x <- tmpdata[1,tmpname]
    if( seriesNames[j] =="brentblend" ){
      tmin1 <- tmpdata[1,"aar"] - 1
      xtmin1 <- brentblendnok[as.character(tmin1)]
      x <- as.numeric( 100*(x/xtmin1 - 1) )
    }
    FINforecasts$t[i,seriesNames[j]] <- x

    ##t pluss 1:
    if( c(i+1) <= length(fileNames) ){
      x <- tmpdata[2,tmpname]
      if( seriesNames[j] =="brentblend" ){
        xtmin1 <- tmpdata[1,tmpname]
        x <- as.numeric( 100*(x/xtmin1 - 1) )
      }
      FINforecasts$tplus1[c(i+1),seriesNames[j]] <- x
    }

    ##t pluss 2:
    if( c(i+2) <= length(fileNames) ){
      x <- tmpdata[3,tmpname]
      if( seriesNames[j] =="brentblend" ){
        xtmin1 <- tmpdata[2,tmpname]
        x <- as.numeric( 100*(x/xtmin1 - 1) )
      }
      FINforecasts$tplus2[c(i+2),seriesNames[j]] <- x
    }
        
  } #close for(j)  

} #close for(i)


##lag matrise m/faktiske verdier tilpasset FIN-prognoser:
##=======================================================

##lag tom matrise:
FINactuals <- forecastData

##fyll matrise:
for(i in 1:NCOL(FINactuals)){

  ##initir:
  tmpname <- colnames(FINactuals)[i]
  Y <- seriesListY[[tmpname]]
  
  ##start og sluttr:
  t1 <- which( index(Y) == rownames(FINactuals)[1] )
  t2 <- which( index(Y) == rownames(FINactuals)[ NROW(FINactuals) ] )
  
  ##fyll FINactuals:
  FINactuals[,i] <- Y[t1:t2]
  
}

##liste med prognosefeil:
##=======================

FINerrors <- list()
FINerrors$t <- FINactuals - FINforecasts$t
FINerrors$tplus1 <- FINactuals - FINforecasts$tplus1
FINerrors$tplus2 <- FINactuals - FINforecasts$tplus2

##spesialanalyse av LN:
##======================

indx <- as.integer(rownames(FINerrors$t))
errors <- zoo(FINerrors$t[,"lonnsvekst"], order.by=indx)

##plot:
maintxt <- expression(paste("Anslagsfeil (dvs. ", italic(Y)[italic(t)],
  " - ", widehat(italic(Y))[italic(t)], ") for lnnsvekst 1998 - 2022"))
plot(errors, col="blue", xlab="", ylab="Prognosefeil (%-poeng)",
  main=maintxt)
abline(h=c(-0.5,0,0.5,1,1.5), col="grey", lty="dashed")
abline(h=0)
abline(v=c(2000, 2005, 2010, 2015, 2020), col="grey", lty="dashed")
lines(errors, col="blue")
points(errors, col="blue", pch=20)

##test uten 1998 og 2020 feilene:
errors <- errors[-c(1,23)]
arx(errors, mc=TRUE, vcov.type="newey-west")


###########################################################
## 4 SSB PROGNOSER
###########################################################

##evalueringsperiode (samme som FIN):
years <- 1998:2022

##filnavn:
fileNames <- paste0("forecasts-ssb-", years, ".csv")
fileNames <- ##legg til katalog:
  paste0("./analysis/data/forecasts-historical/ssb/",
  fileNames)

##tom matrise m/prognoser:
forecastData <- matrix(NA, length(years), length(seriesNames))                     
colnames(forecastData) <- seriesNames
rownames(forecastData) <- years

##liste m/SSB prognoser (t, t+1 og t+2):
SSBforecasts <- list()
SSBforecasts$t <- forecastData
SSBforecasts$tplus1 <- forecastData
SSBforecasts$tplus2 <- forecastData

##lag brentblendnok (faktiske verdier):
brentblendusd <-
  zoo(actualDataY[,"brentblend"], order.by=actualDataY[,"aar"])
nokusd <- 
  zoo(actualDataY[ ,"nokusd" ], order.by=actualDataY[,"aar"])
brentblendnok <- nokusd*brentblendusd

##lkke:
for(i in 1:length(fileNames)){

  ##last data:
  tmpdata <- read.csv(fileNames[i])

  ##fyll prognoser:
#  for(j in 1:2 ){
  for(j in 1:length(seriesNames) ){

    ##navn:
    tmpname <- paste0("SSB", seriesNames[j])
    if( seriesNames[j] =="i44" ){ tmpname <- "SSBi44vekst" }
    if( seriesNames[j] =="brentblend" ){ tmpname <- "SSBbrentblendnok" }

    ##t:
    x <- tmpdata[1,tmpname]
    if( seriesNames[j] =="brentblend" ){
      tmin1 <- tmpdata[1,"aar"] - 1
      xtmin1 <- brentblendnok[as.character(tmin1)]
      x <- as.numeric( 100*(x/xtmin1 - 1) )
    }
    SSBforecasts$t[i,seriesNames[j]] <- x

    ##t pluss 1:
    if( c(i+1) <= length(fileNames) ){
      x <- tmpdata[2,tmpname]
      if( seriesNames[j] =="brentblend" ){
        xtmin1 <- tmpdata[1,tmpname]
        x <- as.numeric( 100*(x/xtmin1 - 1) )
      }
      SSBforecasts$tplus1[c(i+1),seriesNames[j]] <- x
    }

    ##t pluss 2:
    if( c(i+2) <= length(fileNames) ){
      x <- tmpdata[3,tmpname]
      if( seriesNames[j] =="brentblend" ){
        xtmin1 <- tmpdata[2,tmpname]
        x <- as.numeric( 100*(x/xtmin1 - 1) )
      }
      SSBforecasts$tplus2[c(i+2),seriesNames[j]] <- x
    }
        
  } #close for(j)  

} #close for(i)


##lag matrise m/faktiske verdier tilpasset FIN-prognoser:
##=======================================================

##lag tom matrise:
SSBactuals <- forecastData

##fyll matrise:
for(i in 1:NCOL(SSBactuals)){

  ##initir:
  tmpname <- colnames(SSBactuals)[i]
  Y <- seriesListY[[tmpname]]
  
  ##start og sluttr:
  t1 <- which( index(Y) == rownames(SSBactuals)[1] )
  t2 <- which( index(Y) == rownames(SSBactuals)[ NROW(SSBactuals) ] )
  
  ##fyll FINactuals:
  SSBactuals[,i] <- Y[t1:t2]
  
}

##liste med prognosefeil:
##=======================

SSBerrors <- list()
SSBerrors$t <- SSBactuals - SSBforecasts$t
SSBerrors$tplus1 <- SSBactuals - SSBforecasts$tplus1
SSBerrors$tplus2 <- SSBactuals - SSBforecasts$tplus2


###########################################################
## 5 NORGES BANK PROGNOSER
###########################################################

##evalueringsperiode (samme som FIN):
years <- 1998:2022

##filnavn:
fileNames <- paste0("forecasts-norges-bank-", years, ".csv")
fileNames <- ##legg til katalog:
  paste0("./analysis/data/forecasts-historical/norges-bank/",
  fileNames)

##tom matrise m/prognoser:
forecastData <- matrix(NA, length(years), length(seriesNames))                     
colnames(forecastData) <- seriesNames
rownames(forecastData) <- years

##liste m/Norges Bank prognoser (t, t+1 og t+2):
NBforecasts <- list()
NBforecasts$t <- forecastData
NBforecasts$tplus1 <- forecastData
NBforecasts$tplus2 <- forecastData

##lag i44 indeks:
i44 <- zoo(actualDataY[,"i44"], order.by=actualDataY[,"aar"])

##lag brentblendusd:
brentblendusd <-
  zoo(actualDataY[,"brentblend"], order.by=actualDataY[,"aar"])

##fyll NBforecasts matriser:
for(i in 1:length(fileNames)){

  ##last data:
  tmpdata <- read.csv(fileNames[i])

  ##fyll prognoser:
  for(j in 1:length(seriesNames) ){

    ##navn:
    tmpname <- paste0("NB", seriesNames[j])
    if( tmpname == "NBbrentblend" ){ tmpname <- "NBbrentblendusd" }

    ##t:
    x <- tmpdata[1,tmpname]
    if( seriesNames[j] =="i44" ){
      tmin1 <- tmpdata[1,"aar"] - 1
      xtmin1 <- i44[as.character(tmin1)]
      x <- as.numeric( 100*(x/xtmin1 - 1) )
    }
    if( seriesNames[j] =="brentblend" ){
      tmin1 <- tmpdata[1,"aar"] - 1
      xtmin1 <- brentblendusd[as.character(tmin1)]
      x <- as.numeric( 100*(x/xtmin1 - 1) )
    }
    NBforecasts$t[i,seriesNames[j]] <- x

    ##t pluss 1:
    if( c(i+1) <= length(fileNames) ){
      x <- tmpdata[2,tmpname]
      if( seriesNames[j] =="i44" ){
        xtmin1 <- tmpdata[1,tmpname]
        x <- as.numeric( 100*(x/xtmin1 - 1) )
      }
      if( seriesNames[j] =="brentblend" ){
        xtmin1 <- tmpdata[1,tmpname]
        x <- as.numeric( 100*(x/xtmin1 - 1) )
      }
      NBforecasts$tplus1[c(i+1),seriesNames[j]] <- x
    }

    ##t pluss 2:
    if( c(i+2) <= length(fileNames) ){
      x <- tmpdata[3,tmpname]
      if( seriesNames[j] =="i44" ){
        xtmin1 <- tmpdata[2,tmpname]
        x <- as.numeric( 100*(x/xtmin1 - 1) )
      }
      if( seriesNames[j] =="brentblend" ){
        xtmin1 <- tmpdata[2,tmpname]
        x <- as.numeric( 100*(x/xtmin1 - 1) )
      }
      NBforecasts$tplus2[c(i+2),seriesNames[j]] <- x
    }
        
  } #close for(j)  

} #close for(i)


##lag matrise m/faktiske verdier tilpasset NB-prognoser:
##======================================================

##lag tom matrise:
NBactuals <- forecastData

##fyll matrise:
for(i in 1:NCOL(NBactuals)){

  ##initir:
  tmpname <- colnames(NBactuals)[i]
  Y <- seriesListY[[tmpname]]
  
  ##start og sluttr:
  t1 <- which( index(Y) == rownames(NBactuals)[1] )
  t2 <- which( index(Y) == rownames(NBactuals)[ NROW(NBactuals) ] )
  
  ##fyll FINactuals:
  NBactuals[,i] <- Y[t1:t2]
  
}

##liste med prognosefeil:
##=======================

NBerrors <- list()
NBerrors$t <- NBactuals - NBforecasts$t
NBerrors$tplus1 <- NBactuals - NBforecasts$tplus1
NBerrors$tplus2 <- NBactuals - NBforecasts$tplus2


###########################################################
## 6 SAMMENLIGN PROGNOSER
###########################################################

##test FIN for "rasjonalitet" (snittfeil=0):
##==========================================

FINtestst <- list()
FINteststplus1 <- list()
FINteststplus2 <- list()

for(i in 1:NCOL(FINactuals)){
  ##t:
  y <- na.omit(FINerrors$t[,i])
  tmp <- arx(y, mc=TRUE, vcov.type="newey-west")
  FINtestst[[i]] <- tmp

  ##t pluss 1:
  y <- na.omit(FINerrors$tplus1[,i])
  tmp <- arx(y, mc=TRUE, vcov.type="newey-west")
  FINteststplus1[[i]] <- tmp

  ##t pluss 2:
  y <- na.omit(FINerrors$tplus2[,i])
  tmp <- arx(y, mc=TRUE, vcov.type="newey-west")
  FINteststplus2[[i]] <- tmp

}

names(FINtestst) <- colnames(FINactuals)
names(FINteststplus1) <- colnames(FINactuals)
names(FINteststplus2) <- colnames(FINactuals)


##test SSB for "rasjonalitet" (snittfeil=0):
##==========================================

SSBtestst <- list()
SSBteststplus1 <- list()
SSBteststplus2 <- list()

for(i in 1:NCOL(SSBactuals)){
  ##t:
  y <- na.omit(SSBerrors$t[,i])
  tmp <- arx(y, mc=TRUE, vcov.type="newey-west")
  SSBtestst[[i]] <- tmp

  ##t pluss 1:
  y <- na.omit(SSBerrors$tplus1[,i])
  tmp <- arx(y, mc=TRUE, vcov.type="newey-west")
  SSBteststplus1[[i]] <- tmp

  ##t pluss 2:
  y <- na.omit(SSBerrors$tplus2[,i])
  tmp <- arx(y, mc=TRUE, vcov.type="newey-west")
  SSBteststplus2[[i]] <- tmp

}

names(SSBtestst) <- colnames(SSBactuals)
names(SSBteststplus1) <- colnames(SSBactuals)
names(SSBteststplus2) <- colnames(SSBactuals)


##test NB for "rasjonalitet" (snittfeil=0):
##==========================================

##lag liste:
NBtestst <- list()
for(i in 1:length(FINtestst)){ NBtestst[[i]] <- 0 }
names(NBtestst) <- seriesNames
NBteststplus1 <- NBtestst
NBteststplus2 <- NBtestst

##fyll liste:
for( i in 1:length(NBtestst) ){
  ##t:
  y <- na.omit(NBerrors$t[,i])
  if( length(y) > 5 ){
    tmp <- arx(y, mc=TRUE, vcov.type="newey-west")
    NBtestst[[i]] <- tmp
  }

  ##t pluss 1:
  y <- na.omit(NBerrors$tplus1[,i])
  if( length(y) > 5 ){
    tmp <- arx(y, mc=TRUE, vcov.type="newey-west")
    NBteststplus1[[i]] <- tmp
  }
  
  ##t pluss 2:
  y <- na.omit(NBerrors$tplus2[,i])
  if( length(y) > 5 ){
    tmp <- arx(y, mc=TRUE, vcov.type="newey-west")
    NBteststplus2[[i]] <- tmp
  }
}


###########################################################
## 7 LAG TABELLER (LaTeX kode)
###########################################################

##convenience function for the generation of
##LaTeX code (x=coef, y=p-value, n=nobs):
totex <- function(x, y, n, x.digits=2, y.digits=2, dec=NULL){
  result <- rep(NA, length(x))
  for(i in 1:length(x)){
    tmp <- paste0("$\\underset{ \\underset{(n=",
      n, ")}{[",
      format(round(y[i], digits=y.digits), nsmall=y.digits), "]} }{",
      format(round(x[i], digits=x.digits), nsmall=x.digits), "}$")
    result[i] <- tmp 
  }
  if( !is.null(dec) ){ result <- gsub("[.]", dec, result) }
  return(result)
}

##T+1 ("nowcasts"):
##=================

##fin.dep.:
FINtexcode <- character(0)
for(i in 1:length(FINtestst)){
  x <- FINtestst[[i]]$mean.results[1,"coef"]
  y <- FINtestst[[i]]$mean.results[1,"p-value"]
  n <- FINtestst[[i]]$n
  FINtexcode <- paste0(FINtexcode, " & ", totex(x, y, n, dec="{,}"))
}
FINtexcode <- paste0(FINtexcode, " \\\\ ")
cat(FINtexcode)

##SSB:
SSBtexcode <- character(0)
for(i in 1:length(SSBtestst)){
  x <- SSBtestst[[i]]$mean.results[1,"coef"]
  y <- SSBtestst[[i]]$mean.results[1,"p-value"]
  n <- SSBtestst[[i]]$n
  SSBtexcode <- paste0(SSBtexcode, " & ", totex(x, y, n, dec="{,}"))
}
SSBtexcode <- paste0(SSBtexcode, " \\\\ ")
cat(SSBtexcode)

##Norges Bank:
NBtexcode <- character(0)
for(i in 1:length(NBtestst)){
  if( class(NBtestst[[i]]) == "arx" ){
    x <- NBtestst[[i]]$mean.results[1,"coef"]
    y <- NBtestst[[i]]$mean.results[1,"p-value"]
    n <- NBtestst[[i]]$n
  }else{
    x <- NA; y <- NA; n <- NA
  }
  NBtexcode <- paste0(NBtexcode, " & ", totex(x, y, n, dec="{,}"))
}
NBtexcode <- paste0(NBtexcode, " \\\\ ")
cat(NBtexcode)


##T+2:
##====

##fin.dep.:
FINtexcode <- character(0)
for(i in 1:length(FINteststplus1)){
  x <- FINteststplus1[[i]]$mean.results[1,"coef"]
  y <- FINteststplus1[[i]]$mean.results[1,"p-value"]
  n <- FINteststplus1[[i]]$n
  FINtexcode <- paste0(FINtexcode, " & ", totex(x, y, n, dec="{,}"))
}
FINtexcode <- paste0(FINtexcode, " \\\\ ")
cat(FINtexcode)

##SSB:
SSBtexcode <- character(0)
for(i in 1:length(SSBteststplus1)){
  x <- SSBteststplus1[[i]]$mean.results[1,"coef"]
  y <- SSBteststplus1[[i]]$mean.results[1,"p-value"]
  n <- SSBteststplus1[[i]]$n
  SSBtexcode <- paste0(SSBtexcode, " & ", totex(x, y, n, dec="{,}"))
}
SSBtexcode <- paste0(SSBtexcode, " \\\\ ")
cat(SSBtexcode)

##Norges Bank:
NBtexcode <- character(0)
for(i in 1:length(NBteststplus1)){
  if( class(NBteststplus1[[i]]) == "arx" ){
    x <- NBteststplus1[[i]]$mean.results[1,"coef"]
    y <- NBteststplus1[[i]]$mean.results[1,"p-value"]
    n <- NBteststplus1[[i]]$n
  }else{
    x <- NA; y <- NA; n <- NA
  }
  NBtexcode <- paste0(NBtexcode, " & ", totex(x, y, n, dec="{,}"))
}
NBtexcode <- paste0(NBtexcode, " \\\\ ")
cat(NBtexcode)

##T+3:
##====

##fin.dep.
FINtexcode <- character(0)
for(i in 1:length(FINteststplus2)){
  x <- FINteststplus2[[i]]$mean.results[1,"coef"]
  y <- FINteststplus2[[i]]$mean.results[1,"p-value"]
  n <- FINteststplus2[[i]]$n
  FINtexcode <- paste0(FINtexcode, " & ", totex(x, y, n, dec="{,}"))
}
FINtexcode <- paste0(FINtexcode, " \\\\ ")
cat(FINtexcode)

##SSB:
SSBtexcode <- character(0)
for(i in 1:length(SSBteststplus2)){
  x <- SSBteststplus2[[i]]$mean.results[1,"coef"]
  y <- SSBteststplus2[[i]]$mean.results[1,"p-value"]
  n <- SSBteststplus2[[i]]$n
  SSBtexcode <- paste0(SSBtexcode, " & ", totex(x, y, n, dec="{,}"))
}
SSBtexcode <- paste0(SSBtexcode, " \\\\ ")
cat(SSBtexcode)

##Norges Bank:
NBtexcode <- character(0)
for(i in 1:length(NBteststplus2)){
  if( class(NBteststplus2[[i]]) == "arx" ){
    x <- NBteststplus2[[i]]$mean.results[1,"coef"]
    y <- NBteststplus2[[i]]$mean.results[1,"p-value"]
    n <- NBteststplus2[[i]]$n
  }else{
    x <- NA; y <- NA; n <- NA
  }
  NBtexcode <- paste0(NBtexcode, " & ", totex(x, y, n, dec="{,}"))
}
NBtexcode <- paste0(NBtexcode, " \\\\ ")
cat(NBtexcode)
